@@ -45,7 +45,8 @@ class Function(object):
             return None
 
         stack = ref()
-        assert stack is not None, "Need a reference to the Stack object"
+        assert stack is not None, ("Need a reference to the "
+                                   "StackDefinition object")
         return stack
 
     def validate(self):
@@ -73,9 +74,38 @@ class Function(object):
 
         Return an iterator over any attributes of the specified resource that
         this function references.
+
+        The special value heat.engine.attributes.ALL_ATTRIBUTES may be used to
+        indicate that all attributes of the resource are required.
         """
         return dep_attrs(self.args, resource_name)
 
+    def all_dep_attrs(self):
+        """Return resource, attribute name pairs of all attributes referenced.
+
+        Return an iterator over the resource name, attribute name tuples of
+        all attributes that this function references.
+
+        The special value heat.engine.attributes.ALL_ATTRIBUTES may be used to
+        indicate that all attributes of the resource are required.
+
+        By default this calls the dep_attrs() method, but subclasses can
+        override to provide a more efficient implementation.
+        """
+        # If we are using the default dep_attrs method then it will only
+        # return data from the args anyway
+        if type(self).dep_attrs == Function.dep_attrs:
+            return all_dep_attrs(self.args)
+
+        def res_dep_attrs(resource_name):
+            return six.moves.zip(itertools.repeat(resource_name),
+                                 self.dep_attrs(resource_name))
+
+        resource_names = self.stack.enabled_rsrc_names()
+
+        return itertools.chain.from_iterable(six.moves.map(res_dep_attrs,
+                                                           resource_names))
+
     def __reduce__(self):
         """Return a representation of the function suitable for pickling.
 
@@ -179,9 +209,31 @@ class Macro(Function):
 
         Return an iterator over any attributes of the specified resource that
         this function references.
+
+        The special value heat.engine.attributes.ALL_ATTRIBUTES may be used to
+        indicate that all attributes of the resource are required.
         """
         return dep_attrs(self.parsed, resource_name)
 
+    def all_dep_attrs(self):
+        """Return resource, attribute name pairs of all attributes referenced.
+
+        Return an iterator over the resource name, attribute name tuples of
+        all attributes that this function references.
+
+        The special value heat.engine.attributes.ALL_ATTRIBUTES may be used to
+        indicate that all attributes of the resource are required.
+
+        By default this calls the dep_attrs() method, but subclasses can
+        override to provide a more efficient implementation.
+        """
+        # If we are using the default dep_attrs method then it will only
+        # return data from the transformed parsed args anyway
+        if type(self).dep_attrs == Macro.dep_attrs:
+            return all_dep_attrs(self.parsed)
+
+        return super(Macro, self).all_dep_attrs()
+
     def __reduce__(self):
         """Return a representation of the macro result suitable for pickling.
 
@@ -292,6 +344,29 @@ def dep_attrs(snippet, resource_name):
     return []
 
 
+def all_dep_attrs(snippet):
+    """Iterator over resource, attribute name pairs referenced in a snippet.
+
+    The snippet should be already parsed to insert Function objects where
+    appropriate.
+
+    :returns: an iterator over the resource name, attribute name tuples of all
+    attributes that are referenced in the template snippet.
+    """
+
+    if isinstance(snippet, Function):
+        return snippet.all_dep_attrs()
+
+    elif isinstance(snippet, collections.Mapping):
+        res_attrs = (all_dep_attrs(value) for value in snippet.values())
+        return itertools.chain.from_iterable(res_attrs)
+    elif (not isinstance(snippet, six.string_types) and
+          isinstance(snippet, collections.Iterable)):
+        res_attrs = (all_dep_attrs(value) for value in snippet)
+        return itertools.chain.from_iterable(res_attrs)
+    return []
+
+
 class Invalid(Function):
     """A function for checking condition functions and to force failures.
 
