@@ -16,8 +16,10 @@ import hashlib
 import itertools
 
 from oslo_config import cfg
+from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import six
+from six.moves.urllib import parse as urlparse
 import yaql
 from yaql.language import exceptions
 
@@ -26,6 +28,9 @@ from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import function
 
+
+LOG = logging.getLogger(__name__)
+
 opts = [
     cfg.IntOpt('limit_iterators',
                default=200,
@@ -181,15 +186,30 @@ class GetAttThenSelect(function.Function):
             raise exception.InvalidTemplateReference(resource=resource_name,
                                                      key=path)
 
+    def _attr_path(self):
+        return function.resolve(self._attribute)
+
     def dep_attrs(self, resource_name):
         if self._res_name() == resource_name:
-            attrs = [function.resolve(self._attribute)]
+            try:
+                attrs = [self._attr_path()]
+            except Exception as exc:
+                LOG.debug("Ignoring exception calculating required attributes"
+                          ": %s %s", type(exc).__name__, six.text_type(exc))
+                attrs = []
         else:
             attrs = []
         return itertools.chain(super(GetAttThenSelect,
                                      self).dep_attrs(resource_name),
                                attrs)
 
+    def all_dep_attrs(self):
+        try:
+            attrs = [(self._res_name(), self._attr_path())]
+        except Exception:
+            attrs = []
+        return itertools.chain(function.all_dep_attrs(self.args), attrs)
+
     def dependencies(self, path):
         return itertools.chain(super(GetAttThenSelect,
                                      self).dependencies(path),
@@ -209,9 +229,7 @@ class GetAttThenSelect(function.Function):
                 return
 
         attr = function.resolve(self._attribute)
-        from heat.engine import resource
-        if (type(res).get_attribute == resource.Resource.get_attribute and
-                attr not in res.attributes_schema):
+        if attr not in res.attributes_schema:
             raise exception.InvalidTemplateAttribute(
                 resource=self._resource_name, key=attr)
 
@@ -220,11 +238,6 @@ class GetAttThenSelect(function.Function):
                         r.UPDATE, r.ROLLBACK, r.SNAPSHOT, r.CHECK):
             return True
 
-        # NOTE(sirushtim): Add r.INIT to states above once convergence
-        # is the default.
-        if r.stack.has_cache_data(r.name) and r.action == r.INIT:
-            return True
-
         return False
 
     def result(self):
@@ -265,18 +278,13 @@ class GetAtt(GetAttThenSelect):
         else:
             return None
 
-    def dep_attrs(self, resource_name):
-        if self._res_name() == resource_name:
-            path = function.resolve(self._path_components)
-            attr = [function.resolve(self._attribute)]
-            if path:
-                attrs = [tuple(attr + path)]
-            else:
-                attrs = attr
+    def _attr_path(self):
+        path = function.resolve(self._path_components)
+        attr = function.resolve(self._attribute)
+        if path:
+            return tuple([attr] + path)
         else:
-            attrs = []
-        return itertools.chain(function.dep_attrs(self.args, resource_name),
-                               attrs)
+            return attr
 
 
 class GetAttAllAttributes(GetAtt):
@@ -310,17 +318,10 @@ class GetAttAllAttributes(GetAtt):
             raise TypeError(_('Argument to "%s" must be a list') %
                             self.fn_name)
 
-    def dep_attrs(self, resource_name):
-        """Check if there is no attribute_name defined, return empty chain."""
-        if self._attribute is not None:
-            return super(GetAttAllAttributes, self).dep_attrs(resource_name)
-        elif self._res_name() == resource_name:
-            res = self._resource()
-            attrs = six.iterkeys(res.attributes_schema)
-        else:
-            attrs = []
-        return itertools.chain(function.dep_attrs(self.args,
-                                                  resource_name), attrs)
+    def _attr_path(self):
+        if self._attribute is None:
+            return attributes.ALL_ATTRIBUTES
+        return super(GetAttAllAttributes, self)._attr_path()
 
     def result(self):
         if self._attribute is None:
@@ -359,6 +360,7 @@ class Replace(function.Function):
     """
 
     _strict = False
+    _allow_empty_value = True
 
     def __init__(self, stack, fn_name, args):
         super(Replace, self).__init__(stack, fn_name, args)
@@ -388,15 +390,16 @@ class Replace(function.Function):
         else:
             return mapping, string
 
-    def _validate_replacement(self, value):
+    def _validate_replacement(self, value, param):
         if value is None:
             return ''
 
         if not isinstance(value,
                           (six.string_types, six.integer_types,
                            float, bool)):
-            raise TypeError(_('"%s" params must be strings or numbers') %
-                            self.fn_name)
+            raise TypeError(_('"%(name)s" params must be strings or numbers, '
+                              'param %(param)s is not valid') %
+                            {'name': self.fn_name, 'param': param})
 
         return six.text_type(value)
 
@@ -423,7 +426,8 @@ class Replace(function.Function):
                                 self.fn_name)
 
             remaining_keys = keys[1:]
-            value = self._validate_replacement(mapping[placeholder])
+            value = self._validate_replacement(mapping[placeholder],
+                                               placeholder)
 
             def string_split(s):
                 ss = s.split(placeholder)
@@ -467,13 +471,25 @@ class ReplaceJson(Replace):
     being substituted in.
     """
 
-    def _validate_replacement(self, value):
+    def _validate_replacement(self, value, param):
+
+        def _raise_empty_param_value_error():
+            raise ValueError(
+                _('%(name)s has an undefined or empty value for param '
+                  '%(param)s, must be a defined non-empty value') %
+                {'name': self.fn_name, 'param': param})
+
         if value is None:
-            return ''
+            if self._allow_empty_value:
+                return ''
+            else:
+                _raise_empty_param_value_error()
 
         if not isinstance(value, (six.string_types, six.integer_types,
                                   float, bool)):
             if isinstance(value, (collections.Mapping, collections.Sequence)):
+                if not self._allow_empty_value and len(value) == 0:
+                    _raise_empty_param_value_error()
                 try:
                     return jsonutils.dumps(value, default=None, sort_keys=True)
                 except TypeError:
@@ -486,7 +502,10 @@ class ReplaceJson(Replace):
                 raise TypeError(_('"%s" params must be strings, numbers, '
                                   'list or map.') % self.fn_name)
 
-        return six.text_type(value)
+        ret_value = six.text_type(value)
+        if not self._allow_empty_value and not ret_value:
+            _raise_empty_param_value_error()
+        return ret_value
 
 
 class ReplaceJsonStrict(ReplaceJson):
@@ -496,10 +515,19 @@ class ReplaceJsonStrict(ReplaceJson):
     a ValueError is raised if any of the params are not present in
     the template.
     """
-
     _strict = True
 
 
+class ReplaceJsonVeryStrict(ReplaceJsonStrict):
+    """A function for performing string substituions.
+
+    str_replace_vstrict is identical to the str_replace_strict
+    function, only a ValueError is raised if any of the params are
+    None or empty.
+    """
+    _allow_empty_value = False
+
+
 class GetFile(function.Function):
     """A function for including a file inline.
 
@@ -677,7 +705,7 @@ class MapMerge(function.Function):
 
     And resolves to::
 
-        {"<k1>": "<v2>", "<k2>": "<v3>"}
+        {"<k1>": "<v3>", "<k2>": "<v2>"}
 
     """
 
@@ -860,9 +888,12 @@ class Repeat(function.Function):
     is a copy of <body> with any occurrences of <var> replaced with the
     corresponding item of <list>.
     """
+
     def __init__(self, stack, fn_name, args):
         super(Repeat, self).__init__(stack, fn_name, args)
+        self._parse_args()
 
+    def _parse_args(self):
         if not isinstance(self.args, collections.Mapping):
             raise TypeError(_('Arguments to "%s" must be a map') %
                             self.fn_name)
@@ -879,6 +910,8 @@ class Repeat(function.Function):
                 %var%: ['a', 'b', 'c']''')
             raise KeyError(_('"repeat" syntax should be %s') % example)
 
+        self._nested_loop = True
+
     def validate(self):
         super(Repeat, self).validate()
 
@@ -887,11 +920,12 @@ class Repeat(function.Function):
                 raise TypeError(_('The "for_each" argument to "%s" must '
                                   'contain a map') % self.fn_name)
 
-    @staticmethod
-    def _valid_arg(arg):
-        return (isinstance(arg, (collections.Sequence,
+    def _valid_arg(self, arg):
+        if not (isinstance(arg, (collections.Sequence,
                                  function.Function)) and
-                not isinstance(arg, six.string_types))
+                not isinstance(arg, six.string_types)):
+            raise TypeError(_('The values of the "for_each" argument to '
+                              '"%s" must be lists') % self.fn_name)
 
     def _do_replacement(self, keys, values, template):
         if isinstance(template, six.string_types):
@@ -914,30 +948,97 @@ class Repeat(function.Function):
 
         # use empty list for references(None) else validation will fail
         values = [[] if value is None else value for value in lists]
-
-        if not all(self._valid_arg(l) for l in values):
-            raise TypeError(_('The values of the "for_each" argument to '
-                              '"%s" must be lists') % self.fn_name)
+        value_lens = []
+        for arg in values:
+            self._valid_arg(arg)
+            value_lens.append(len(arg))
+        if not self._nested_loop:
+            if len(set(value_lens)) != 1:
+                raise ValueError(_('For %s, the length of for_each values '
+                                   'should be equal if no nested '
+                                   'loop.') % self.fn_name)
 
         template = function.resolve(self._template)
+        iter_func = itertools.product if self._nested_loop else six.moves.zip
 
         return [self._do_replacement(keys, replacements, template)
-                for replacements in itertools.product(*values)]
+                for replacements in iter_func(*values)]
 
 
 class RepeatWithMap(Repeat):
-    """A function for iterating over a list of items.
+    """A function for iterating over a list of items or a dict of keys.
 
-    Behaves the same as Replace, but if tolerates a map as
-    values to be repeated, in which case it iterates the map keys.
+    Takes the form::
+
+        repeat:
+            template:
+                <body>
+            for_each:
+                <var>: <list> or <dict>
+
+    The result is a new list of the same size as <list> or <dict>, where each
+    element is a copy of <body> with any occurrences of <var> replaced with the
+    corresponding item of <list> or key of <dict>.
     """
 
-    @staticmethod
-    def _valid_arg(arg):
-        return (isinstance(arg, (collections.Sequence,
+    def _valid_arg(self, arg):
+        if not (isinstance(arg, (collections.Sequence,
                                  collections.Mapping,
                                  function.Function)) and
-                not isinstance(arg, six.string_types))
+                not isinstance(arg, six.string_types)):
+            raise TypeError(_('The values of the "for_each" argument to '
+                              '"%s" must be lists or maps') % self.fn_name)
+
+
+class RepeatWithNestedLoop(RepeatWithMap):
+    """A function for iterating over a list of items or a dict of keys.
+
+    Takes the form::
+
+        repeat:
+            template:
+                <body>
+            for_each:
+                <var>: <list> or <dict>
+
+    The result is a new list of the same size as <list> or <dict>, where each
+    element is a copy of <body> with any occurrences of <var> replaced with the
+    corresponding item of <list> or key of <dict>.
+
+    This function also allows to specify 'permutations' to decide
+    whether to iterate nested the over all the permutations of the
+    elements in the given lists.
+
+    Takes the form::
+
+        repeat:
+          template:
+            var: %var%
+            bar: %bar%
+          for_each:
+            %var%: <list1>
+            %bar%: <list2>
+          permutations: false
+
+    If 'permutations' is not specified, we set the default value to true to
+    compatible with before behavior. The args have to be lists instead of
+    dicts if 'permutations' is False because keys in a dict are unordered,
+    and the list args all have to be of the same length.
+    """
+
+    def _parse_args(self):
+        super(RepeatWithNestedLoop, self)._parse_args()
+        self._nested_loop = self.args.get('permutations', True)
+
+        if not isinstance(self._nested_loop, bool):
+            raise TypeError(_('"permutations" should be boolean type '
+                              'for %s function.') % self.fn_name)
+
+    def _valid_arg(self, arg):
+        if self._nested_loop:
+            super(RepeatWithNestedLoop, self)._valid_arg(arg)
+        else:
+            Repeat._valid_arg(self, arg)
 
 
 class Digest(function.Function):
@@ -1320,3 +1421,223 @@ class Filter(function.Function):
             raise TypeError(
                 _('"%(fn)s" filters a list of values') % self.fn_name)
         return [i for i in sequence if i not in values]
+
+
+class MakeURL(function.Function):
+    """A function for performing substitutions on maps.
+
+    Takes the form::
+
+        make_url:
+          scheme: <protocol>
+          username: <username>
+          password: <password>
+          host: <hostname or IP>
+          port: <port>
+          path: <path>
+          query:
+            <key1>: <value1>
+          fragment: <fragment>
+
+    And resolves to a correctly-escaped URL constructed from the various
+    components.
+    """
+
+    _ARG_KEYS = (
+        SCHEME, USERNAME, PASSWORD, HOST, PORT,
+        PATH, QUERY, FRAGMENT,
+    ) = (
+        'scheme', 'username', 'password', 'host', 'port',
+        'path', 'query', 'fragment',
+    )
+
+    def _check_args(self, args):
+        for arg in self._ARG_KEYS:
+            if arg in args:
+                if arg == self.QUERY:
+                    if not isinstance(args[arg], (function.Function,
+                                                  collections.Mapping)):
+                        raise TypeError(_('The "%(arg)s" argument to '
+                                          '"%(fn_name)s" must be a map') %
+                                        {'arg': arg,
+                                         'fn_name': self.fn_name})
+                    return
+                elif arg == self.PORT:
+                    port = args[arg]
+                    if not isinstance(port, function.Function):
+                        if not isinstance(port, six.integer_types):
+                            try:
+                                port = int(port)
+                            except ValueError:
+                                raise ValueError(_('Invalid URL port "%s"') %
+                                                 port)
+                        if not (0 < port <= 65535):
+                            raise ValueError(_('Invalid URL port %d') % port)
+                else:
+                    if not isinstance(args[arg], (function.Function,
+                                                  six.string_types)):
+                        raise TypeError(_('The "%(arg)s" argument to '
+                                          '"%(fn_name)s" must be a string') %
+                                        {'arg': arg,
+                                         'fn_name': self.fn_name})
+
+    def validate(self):
+        super(MakeURL, self).validate()
+
+        if not isinstance(self.args, collections.Mapping):
+            raise TypeError(_('The arguments to "%s" must '
+                              'be a map') % self.fn_name)
+
+        invalid_keys = set(self.args) - set(self._ARG_KEYS)
+        if invalid_keys:
+            raise ValueError(_('Invalid arguments to "%(fn)s": %(args)s') %
+                             {'fn': self.fn_name,
+                              'args': ', '.join(invalid_keys)})
+
+        self._check_args(self.args)
+
+    def result(self):
+        args = function.resolve(self.args)
+        self._check_args(args)
+
+        scheme = args.get(self.SCHEME, '')
+        if ':' in scheme:
+            raise ValueError(_('URL "%s" should not contain \':\'') %
+                             self.SCHEME)
+
+        def netloc():
+            username = urlparse.quote(args.get(self.USERNAME, ''), safe='')
+            password = urlparse.quote(args.get(self.PASSWORD, ''), safe='')
+            if username or password:
+                yield username
+                if password:
+                    yield ':'
+                    yield password
+                yield '@'
+
+            host = args.get(self.HOST, '')
+            if host.startswith('[') and host.endswith(']'):
+                host = host[1:-1]
+            host = urlparse.quote(host, safe=':')
+            if ':' in host:
+                host = '[%s]' % host
+            yield host
+
+            port = args.get(self.PORT, '')
+            if port:
+                yield ':'
+                yield six.text_type(port)
+
+        path = urlparse.quote(args.get(self.PATH, ''))
+
+        query_dict = args.get(self.QUERY, {})
+        query = urlparse.urlencode(query_dict).replace('%2F', '/')
+
+        fragment = urlparse.quote(args.get(self.FRAGMENT, ''))
+
+        return urlparse.urlunsplit((scheme, ''.join(netloc()),
+                                    path, query, fragment))
+
+
+class ListConcat(function.Function):
+    """A function for extending lists.
+
+    Takes the form::
+
+        list_concat:
+          - [<value 1>, <value 2>]
+          - [<value 3>, <value 4>]
+
+    And resolves to::
+
+        [<value 1>, <value 2>, <value 3>, <value 4>]
+
+    """
+
+    _unique = False
+
+    def __init__(self, stack, fn_name, args):
+        super(ListConcat, self).__init__(stack, fn_name, args)
+        example = (_('"%s" : [ [ <value 1>, <value 2> ], '
+                     '[ <value 3>, <value 4> ] ]')
+                   % fn_name)
+        self.fmt_data = {'fn_name': fn_name, 'example': example}
+
+    def result(self):
+        args = function.resolve(self.args)
+
+        if (isinstance(args, six.string_types) or
+                not isinstance(args, collections.Sequence)):
+            raise TypeError(_('Incorrect arguments to "%(fn_name)s" '
+                              'should be: %(example)s') % self.fmt_data)
+
+        def ensure_list(m):
+            if m is None:
+                return []
+            elif (isinstance(m, collections.Sequence) and
+                  not isinstance(m, six.string_types)):
+                return m
+            else:
+                msg = _('Incorrect arguments: Items to concat must be lists.')
+                raise TypeError(msg)
+
+        ret_list = []
+        for m in args:
+            ret_list.extend(ensure_list(m))
+
+        if self._unique:
+            for i in ret_list:
+                while ret_list.count(i) > 1:
+                    del ret_list[ret_list.index(i)]
+
+        return ret_list
+
+
+class ListConcatUnique(ListConcat):
+    """A function for extending lists with unique items.
+
+    list_concat_unique is identical to the list_concat function, only
+    contains unique items in retuning list.
+    """
+
+    _unique = True
+
+
+class Contains(function.Function):
+    """A function for checking whether specific value is in sequence.
+
+    Takes the form::
+
+        contains:
+          - <value>
+          - <sequence>
+
+    The value can be any type that you want to check. Returns true
+    if the specific value is in the sequence, otherwise returns false.
+    """
+
+    def __init__(self, stack, fn_name, args):
+        super(Contains, self).__init__(stack, fn_name, args)
+        example = '"%s" : [ "value1", [ "value1", "value2"]]' % self.fn_name
+        fmt_data = {'fn_name': self.fn_name,
+                    'example': example}
+
+        if not self.args or not isinstance(self.args, list):
+            raise TypeError(_('Incorrect arguments to "%(fn_name)s" '
+                              'should be: %(example)s') % fmt_data)
+        try:
+            self.value, self.sequence = self.args
+        except ValueError:
+            msg = _('Arguments to "%s" must be of the form: '
+                    '[value1, [value1, value2]]')
+            raise ValueError(msg % self.fn_name)
+
+    def result(self):
+        resolved_value = function.resolve(self.value)
+        resolved_sequence = function.resolve(self.sequence)
+
+        if not isinstance(resolved_sequence, collections.Sequence):
+            raise TypeError(_('Second argument to "%s" should be '
+                              'a sequence.') % self.fn_name)
+
+        return resolved_value in resolved_sequence
