@@ -11,6 +11,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import functools
+import six
+
 from heat.common import environment_format
 from heat.common import grouputils
 from heat.common.i18n import _
@@ -18,7 +21,7 @@ from heat.common import short_id
 from heat.common import timeutils as iso8601utils
 from heat.engine import attributes
 from heat.engine import environment
-from heat.engine import function
+from heat.engine import output
 from heat.engine import properties
 from heat.engine.resources import stack_resource
 from heat.engine import rsrc_defn
@@ -225,10 +228,16 @@ class InstanceGroup(stack_resource.StackResource):
     def _get_conf_properties(self):
         conf_refid = self.properties[self.LAUNCH_CONFIGURATION_NAME]
         conf = self.stack.resource_by_refid(conf_refid)
-        props = function.resolve(conf.properties.data)
+        c_props = conf.frozen_definition().properties(conf.properties_schema,
+                                                      conf.context)
+        props = {k: v for k, v in c_props.items() if k in c_props.data}
+        for key in [conf.BLOCK_DEVICE_MAPPINGS, conf.NOVA_SCHEDULER_HINTS]:
+            if props.get(key) is not None:
+                props[key] = [{k: v for k, v in prop.items()
+                               if k in c_props.data[key][idx]}
+                              for idx, prop in enumerate(props[key])]
         if 'InstanceId' in props:
             props = conf.rebuild_lc_properties(props['InstanceId'])
-
         props['Tags'] = self._tags()
         # if the launch configuration is created from an existing instance.
         # delete the 'InstanceId' property
@@ -253,17 +262,28 @@ class InstanceGroup(stack_resource.StackResource):
         instance_definition = self._get_resource_definition()
         old_resources = grouputils.get_member_definitions(self,
                                                           include_failed=True)
-        definitions = template.member_definitions(
+        definitions = list(template.member_definitions(
             old_resources, instance_definition, num_instances, num_replace,
-            short_id.generate_id)
+            short_id.generate_id))
 
         child_env = environment.get_child_environment(
             self.stack.env,
             self.child_params(), item_to_remove=self.resource_info)
 
-        return template.make_template(definitions, version=template_version,
+        tmpl = template.make_template(definitions, version=template_version,
                                       child_env=child_env)
 
+        # Subclasses use HOT templates
+        att_func = 'get_attr'
+        if att_func not in tmpl.functions:
+            att_func = 'Fn::GetAtt'
+        get_attr = functools.partial(tmpl.functions[att_func], None, att_func)
+        for odefn in self._nested_output_defns([k for k, d in definitions],
+                                               get_attr):
+            tmpl.add_output(odefn)
+
+        return tmpl
+
     def _try_rolling_update(self, prop_diff):
         if (self.update_policy[self.ROLLING_UPDATE] and
                 self.LAUNCH_CONFIGURATION_NAME in prop_diff):
@@ -289,7 +309,7 @@ class InstanceGroup(stack_resource.StackResource):
         def changing_instances(tmpl):
             instances = grouputils.get_members(self)
             current = set((i.name, i.t) for i in instances)
-            updated = set(tmpl.resource_definitions(self.nested()).items())
+            updated = set(tmpl.resource_definitions(None).items())
             # includes instances to be updated and deleted
             affected = set(k for k, v in current ^ updated)
             return set(i.FnGetRefId() for i in instances if i.name in affected)
@@ -377,9 +397,20 @@ class InstanceGroup(stack_resource.StackResource):
         ip addresses.
         """
         if name == self.INSTANCE_LIST:
-            return u','.join(inst.FnGetAtt('PublicIp')
+            return u','.join(inst.FnGetAtt('PublicIp') or '0.0.0.0'
                              for inst in grouputils.get_members(self)) or None
 
+    def _nested_output_defns(self, resource_names, get_attr_fn):
+        for attr in self.referenced_attrs():
+            if isinstance(attr, six.string_types):
+                key = attr
+            else:
+                key = attr[0]
+
+            if key == self.INSTANCE_LIST:
+                value = [get_attr_fn([r, 'PublicIp']) for r in resource_names]
+                yield output.OutputDefinition(key, value)
+
     def child_template(self):
         num_instances = int(self.properties[self.SIZE])
         return self._create_template(num_instances)
