@@ -142,7 +142,7 @@ class StructuredDeployment(sd.SoftwareDeployment):
         input_key = self.properties[self.INPUT_KEY]
         check_input_val = self.properties[self.INPUT_VALUES_VALIDATE]
 
-        inputs = dict((i['name'], i['value']) for i in derived_inputs)
+        inputs = dict(i.input_data() for i in derived_inputs)
 
         return self.parse(inputs, input_key, cfg, check_input_val)
 
@@ -235,11 +235,15 @@ class StructuredDeploymentGroup(sd.SoftwareDeploymentGroup):
 
 class StructuredDeployments(StructuredDeploymentGroup):
 
-    deprecation_msg = _('Use of this resource is discouraged. Please use '
-                        'OS::Heat::StructuredDeploymentGroup instead.')
-    support_status = support.SupportStatus(status=support.DEPRECATED,
-                                           message=deprecation_msg,
-                                           version='2014.2')
+    hidden_msg = _('Please use OS::Heat::StructuredDeploymentGroup instead.')
+    support_status = support.SupportStatus(
+        status=support.HIDDEN,
+        message=hidden_msg,
+        version='7.0.0',
+        previous_status=support.SupportStatus(
+            status=support.DEPRECATED,
+            version='2014.2'),
+        substitute_class=StructuredDeploymentGroup)
 
 
 def resource_mapping():
