@@ -21,7 +21,6 @@ from oslo_utils import encodeutils
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
@@ -118,6 +117,7 @@ class SaharaNodeGroupTemplate(resource.Resource):
             constraints=[
                 constraints.Range(min=0),
             ],
+            default=0,
             update_allowed=True
         ),
         VOLUMES_SIZE: properties.Schema(
@@ -285,7 +285,7 @@ class SaharaNodeGroupTemplate(resource.Resource):
         props = dict((k, v) for k, v in six.iteritems(self.properties))
         args = self._prepare_properties(props)
         node_group_template = self.client().node_group_templates.create(**args)
-        LOG.info(_LI("Node Group Template '%s' has been created"),
+        LOG.info("Node Group Template '%s' has been created",
                  node_group_template.name)
         self.resource_id_set(node_group_template.id)
         return self.resource_id
@@ -351,6 +351,17 @@ class SaharaNodeGroupTemplate(resource.Resource):
                       self.stack.t.get_section_name('properties')],
                 message=msg)
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        result = super(SaharaNodeGroupTemplate, self).parse_live_resource_data(
+            resource_properties, resource_data)
+
+        for group in result[self.SHARES] or []:
+            remove_keys = set(group.keys()) - set(self._SHARE_KEYS)
+            for key in remove_keys:
+                del group[key]
+        result[self.FLAVOR] = resource_data.get('flavor_id')
+        return result
+
 
 class SaharaClusterTemplate(resource.Resource):
     """A resource for managing Sahara cluster templates.
@@ -525,15 +536,6 @@ class SaharaClusterTemplate(resource.Resource):
     entity = 'cluster_templates'
 
     def translation_rules(self, props):
-        if not self.is_using_neutron():
-            return [
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.RESOLVE,
-                    [self.MANAGEMENT_NETWORK],
-                    client_plugin=self.client_plugin('nova'),
-                    finder='get_nova_network_id')
-                ]
         return [
             translation.TranslationRule(
                 props,
@@ -541,8 +543,7 @@ class SaharaClusterTemplate(resource.Resource):
                 [self.MANAGEMENT_NETWORK],
                 client_plugin=self.client_plugin('neutron'),
                 finder='find_resourceid_by_name_or_id',
-                entity='network')
-            ]
+                entity='network')]
 
     def _cluster_template_name(self, name):
         if name:
@@ -561,7 +562,7 @@ class SaharaClusterTemplate(resource.Resource):
         props = dict((k, v) for k, v in six.iteritems(self.properties))
         args = self._prepare_properties(props)
         cluster_template = self.client().cluster_templates.create(**args)
-        LOG.info(_LI("Cluster Template '%s' has been created"),
+        LOG.info("Cluster Template '%s' has been created",
                  cluster_template.name)
         self.resource_id_set(cluster_template.id)
         return self.resource_id
@@ -587,6 +588,20 @@ class SaharaClusterTemplate(resource.Resource):
             self.properties[self.HADOOP_VERSION]
         )
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        result = super(SaharaClusterTemplate, self).parse_live_resource_data(
+            resource_properties, resource_data)
+
+        for group in result[self.NODE_GROUPS] or []:
+            remove_keys = set(group.keys()) - set(self._NODE_GROUP_KEYS)
+            for key in remove_keys:
+                del group[key]
+        for group in result[self.SHARES] or []:
+            remove_keys = set(group.keys()) - set(self._SHARE_KEYS)
+            for key in remove_keys:
+                del group[key]
+        return result
+
 
 def resource_mapping():
     return {
