@@ -99,6 +99,8 @@ class ResourceDefinition(object):
 
         self._hash = hash(self.resource_type)
         self._rendering = None
+        self._dep_names = None
+        self._all_dep_attrs = None
 
         assert isinstance(self.description, six.string_types)
 
@@ -138,6 +140,8 @@ class ResourceDefinition(object):
                                           function.Function))
             self._hash ^= _hash_data(condition)
 
+        self.set_translation_rules()
+
     def freeze(self, **overrides):
         """Return a frozen resource definition, with all functions resolved.
 
@@ -189,22 +193,66 @@ class ResourceDefinition(object):
             external_id=reparse_snippet(self._external_id),
             condition=self._condition)
 
-    def dep_attrs(self, resource_name):
+    def dep_attrs(self, resource_name, load_all=False):
         """Iterate over attributes of a given resource that this references.
 
         Return an iterator over dependent attributes for specified
         resource_name in resources' properties and metadata fields.
         """
+        if self._all_dep_attrs is None and load_all:
+            attr_map = collections.defaultdict(set)
+            atts = itertools.chain(function.all_dep_attrs(self._properties),
+                                   function.all_dep_attrs(self._metadata))
+            for res_name, att_name in atts:
+                attr_map[res_name].add(att_name)
+            self._all_dep_attrs = attr_map
+
+        if self._all_dep_attrs is not None:
+            return self._all_dep_attrs[resource_name]
+
         return itertools.chain(function.dep_attrs(self._properties,
                                                   resource_name),
                                function.dep_attrs(self._metadata,
                                                   resource_name))
 
+    def required_resource_names(self):
+        """Return a set of names of all resources on which this depends.
+
+        Note that this is done entirely in isolation from the rest of the
+        template, so the resource names returned may refer to resources that
+        don't actually exist, or would have strict_dependency=False. Use the
+        dependencies() method to get validated dependencies.
+        """
+        if self._dep_names is None:
+            explicit_depends = [] if self._depends is None else self._depends
+
+            def path(section):
+                return '.'.join([self.name, section])
+
+            prop_deps = function.dependencies(self._properties,
+                                              path('Properties'))
+            metadata_deps = function.dependencies(self._metadata,
+                                                  path('Metadata'))
+            implicit_depends = six.moves.map(lambda rp: rp.name,
+                                             itertools.chain(prop_deps,
+                                                             metadata_deps))
+
+            # (ricolin) External resource should not depend on any other
+            # resources. This operation is not allowed for now.
+            if self.external_id():
+                if explicit_depends:
+                    raise exception.InvalidExternalResourceDependency(
+                        external_id=self.external_id(),
+                        resource_type=self.resource_type
+                    )
+                self._dep_names = set()
+            else:
+                self._dep_names = set(itertools.chain(explicit_depends,
+                                                      implicit_depends))
+        return self._dep_names
+
     def dependencies(self, stack):
         """Return the Resource objects in given stack on which this depends."""
-        def path(section):
-            return '.'.join([self.name, section])
-
         def get_resource(res_name):
             if res_name not in stack:
                 if res_name in stack.t.get(stack.t.RESOURCES):
@@ -213,30 +261,18 @@ class ResourceDefinition(object):
                     return
                 raise exception.InvalidTemplateReference(resource=res_name,
                                                          key=self.name)
-            return stack[res_name]
+            res = stack[res_name]
+            if getattr(res, 'strict_dependency', True):
+                return res
 
-        def strict_func_deps(data, datapath):
-            return six.moves.filter(
-                lambda r: getattr(r, 'strict_dependency', True),
-                function.dependencies(data, datapath))
+        return six.moves.filter(None,
+                                six.moves.map(get_resource,
+                                              self.required_resource_names()))
 
-        explicit_depends = [] if self._depends is None else self._depends
-        prop_deps = strict_func_deps(self._properties, path('Properties'))
-        metadata_deps = strict_func_deps(self._metadata, path('Metadata'))
-
-        # (ricolin) External resource should not depend on any other resources.
-        # This operation is not allowed for now.
-        if self.external_id():
-            if explicit_depends:
-                raise exception.InvalidExternalResourceDependency(
-                    external_id=self.external_id(),
-                    resource_type=self.resource_type
-                )
-            return itertools.chain()
-
-        return itertools.chain(
-            filter(None, (get_resource(dep) for dep in explicit_depends)),
-            prop_deps, metadata_deps)
+    def set_translation_rules(self, rules=None, client_resolve=True):
+        """Helper method to update properties with translation rules."""
+        self._rules = rules or []
+        self._client_resolve = client_resolve
 
     def properties(self, schema, context=None):
         """Return a Properties object representing the resource properties.
@@ -244,9 +280,11 @@ class ResourceDefinition(object):
         The Properties object is constructed from the given schema, and may
         require a context to validate constraints.
         """
-        return properties.Properties(schema, self._properties or {},
-                                     function.resolve, self.name, context,
-                                     section='Properties')
+        props = properties.Properties(schema, self._properties or {},
+                                      function.resolve, context=context,
+                                      section='Properties')
+        props.update_translation(self._rules, self._client_resolve)
+        return props
 
     def deletion_policy(self):
         """Return the deletion policy for the resource.
@@ -261,9 +299,11 @@ class ResourceDefinition(object):
         The Properties object is constructed from the given schema, and may
         require a context to validate constraints.
         """
-        return properties.Properties(schema, self._update_policy or {},
-                                     function.resolve, self.name, context,
-                                     section='UpdatePolicy')
+        props = properties.Properties(schema, self._update_policy or {},
+                                      function.resolve, context=context,
+                                      section='UpdatePolicy')
+        props.update_translation(self._rules, self._client_resolve)
+        return props
 
     def metadata(self):
         """Return the resource metadata."""
