@@ -24,6 +24,7 @@ from heat.engine import function
 from heat.engine import resource
 from heat.engine import rsrc_defn
 from heat.engine import stack
+from heat.engine import stk_defn
 from heat.engine import template
 from heat.tests import common
 from heat.tests import utils
@@ -178,9 +179,9 @@ class ValidateTest(common.HeatTestCase):
     def test_validate_func(self):
         self.assertIsNone(function.validate(self.func))
         self.func = TestFunction(None, 'foo', ['bar'])
-        self.assertRaisesRegexp(exception.StackValidationFailed,
-                                '.foo: Need more arguments',
-                                function.validate, self.func)
+        self.assertRaisesRegex(exception.StackValidationFailed,
+                               '.foo: Need more arguments',
+                               function.validate, self.func)
 
     def test_validate_dict(self):
         snippet = {'foo': 'bar', 'blarg': self.func}
@@ -188,9 +189,9 @@ class ValidateTest(common.HeatTestCase):
 
         self.func = TestFunction(None, 'foo', ['bar'])
         snippet = {'foo': 'bar', 'blarg': self.func}
-        self.assertRaisesRegexp(exception.StackValidationFailed,
-                                '.blarg.foo: Need more arguments',
-                                function.validate, snippet)
+        self.assertRaisesRegex(exception.StackValidationFailed,
+                               '.blarg.foo: Need more arguments',
+                               function.validate, snippet)
 
     def test_validate_list(self):
         snippet = ['foo', 'bar', 'baz', 'blarg', self.func]
@@ -198,9 +199,9 @@ class ValidateTest(common.HeatTestCase):
 
         self.func = TestFunction(None, 'foo', ['bar'])
         snippet = {'foo': 'bar', 'blarg': self.func}
-        self.assertRaisesRegexp(exception.StackValidationFailed,
-                                '.blarg.foo: Need more arguments',
-                                function.validate, snippet)
+        self.assertRaisesRegex(exception.StackValidationFailed,
+                               '.blarg.foo: Need more arguments',
+                               function.validate, snippet)
 
     def test_validate_all(self):
         snippet = ['foo', {'bar': ['baz', {'blarg': self.func}]}]
@@ -208,9 +209,9 @@ class ValidateTest(common.HeatTestCase):
 
         self.func = TestFunction(None, 'foo', ['bar'])
         snippet = {'foo': 'bar', 'blarg': self.func}
-        self.assertRaisesRegexp(exception.StackValidationFailed,
-                                '.blarg.foo: Need more arguments',
-                                function.validate, snippet)
+        self.assertRaisesRegex(exception.StackValidationFailed,
+                               '.blarg.foo: Need more arguments',
+                               function.validate, snippet)
 
 
 class DependenciesTest(common.HeatTestCase):
@@ -241,25 +242,38 @@ class ValidateGetAttTest(common.HeatTestCase):
         env.load({u'resource_registry':
                   {u'OS::Test::FakeResource': u'OverwrittenFnGetAttType'}})
 
+        tmpl = template.Template({"HeatTemplateFormatVersion": "2012-12-12",
+                                  "Resources": {
+                                      "test_rsrc": {
+                                          "Type": "OS::Test::GenericResource"
+                                      },
+                                      "get_att_rsrc": {
+                                          "Type": "OS::Heat::Value",
+                                          "Properties": {
+                                              "value": {
+                                                  "Fn::GetAtt": ["test_rsrc",
+                                                                 "Foo"]
+                                              }
+                                          }
+                                      }
+                                  }},
+                                 env=env)
         self.stack = stack.Stack(
             utils.dummy_context(), 'test_stack',
-            template.Template({"HeatTemplateFormatVersion": "2012-12-12"},
-                              env=env),
+            tmpl,
             stack_id=str(uuid.uuid4()))
-        res_defn = rsrc_defn.ResourceDefinition('test_rsrc',
-                                                'OS::Test::GenericResource')
-        self.rsrc = resource.Resource('test_rsrc', res_defn, self.stack)
-        self.stack.add_resource(self.rsrc)
+        self.rsrc = self.stack['test_rsrc']
+        self.stack.validate()
 
     def test_resource_is_appear_in_stack(self):
-        func = functions.GetAtt(self.stack, 'Fn::GetAtt',
+        func = functions.GetAtt(self.stack.defn, 'Fn::GetAtt',
                                 [self.rsrc.name, 'Foo'])
         self.assertIsNone(func.validate())
 
     def test_resource_is_not_appear_in_stack(self):
         self.stack.remove_resource(self.rsrc.name)
 
-        func = functions.GetAtt(self.stack, 'Fn::GetAtt',
+        func = functions.GetAtt(self.stack.defn, 'Fn::GetAtt',
                                 [self.rsrc.name, 'Foo'])
         ex = self.assertRaises(exception.InvalidTemplateReference,
                                func.validate)
@@ -267,7 +281,15 @@ class ValidateGetAttTest(common.HeatTestCase):
                          'is incorrect.', six.text_type(ex))
 
     def test_resource_no_attribute_with_default_fn_get_att(self):
-        func = functions.GetAtt(self.stack, 'Fn::GetAtt',
+        res_defn = rsrc_defn.ResourceDefinition('test_rsrc',
+                                                'ResWithStringPropAndAttr')
+        self.rsrc = resource.Resource('test_rsrc', res_defn, self.stack)
+        self.stack.add_resource(self.rsrc)
+        stk_defn.update_resource_data(self.stack.defn, self.rsrc.name,
+                                      self.rsrc.node_data())
+        self.stack.validate()
+
+        func = functions.GetAtt(self.stack.defn, 'Fn::GetAtt',
                                 [self.rsrc.name, 'Bar'])
         ex = self.assertRaises(exception.InvalidTemplateAttribute,
                                func.validate)
@@ -278,16 +300,19 @@ class ValidateGetAttTest(common.HeatTestCase):
         res_defn = rsrc_defn.ResourceDefinition('test_rsrc',
                                                 'OS::Test::FakeResource')
         self.rsrc = resource.Resource('test_rsrc', res_defn, self.stack)
-        self.stack.add_resource(self.rsrc)
         self.rsrc.attributes_schema = {}
+        self.stack.add_resource(self.rsrc)
+        stk_defn.update_resource_data(self.stack.defn, self.rsrc.name,
+                                      self.rsrc.node_data())
+        self.stack.validate()
 
-        func = functions.GetAtt(self.stack, 'Fn::GetAtt',
+        func = functions.GetAtt(self.stack.defn, 'Fn::GetAtt',
                                 [self.rsrc.name, 'Foo'])
         self.assertIsNone(func.validate())
 
     def test_get_attr_without_attribute_name(self):
         ex = self.assertRaises(ValueError, functions.GetAtt,
-                               self.stack, 'Fn::GetAtt', [self.rsrc.name])
+                               self.stack.defn, 'Fn::GetAtt', [self.rsrc.name])
         self.assertEqual('Arguments to "Fn::GetAtt" must be '
                          'of the form [resource_name, attribute]',
                          six.text_type(ex))
