@@ -85,7 +85,7 @@ class StackTest(common.HeatTestCase):
             self.ctx, stack_id))
 
     def test_delete_with_snapshot_after_stack_add_resource(self):
-        tpl = {'heat_template_version': 'ocata',
+        tpl = {'heat_template_version': 'pike',
                'resources':
                    {'A': {'type': 'ResourceWithRestoreType'}}}
         self.stack = stack.Stack(self.ctx, 'stack_delete_with_snapshot',
@@ -107,7 +107,7 @@ class StackTest(common.HeatTestCase):
         self.assertIsNotNone(snapshot_object.Snapshot.get_all(
             self.ctx, stack_id))
 
-        new_tmpl = {'heat_template_version': 'ocata',
+        new_tmpl = {'heat_template_version': 'pike',
                     'resources':
                         {'A': {'type': 'ResourceWithRestoreType'},
                          'B': {'type': 'ResourceWithRestoreType'}}}
@@ -473,8 +473,8 @@ class StackTest(common.HeatTestCase):
     def test_delete_stack_with_resource_log_is_clear(self):
         debug_logger = self.useFixture(
             fixtures.FakeLogger(level=logging.DEBUG,
-                                format="%(levelname)8s [%(name)s] %("
-                                       "message)s"))
+                                format="%(levelname)8s [%(name)s] "
+                                       "%(message)s"))
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {'AResource': {'Type': 'GenericResourceType'}}}
         self.stack = stack.Stack(self.ctx, 'delete_log_test',
