@@ -55,11 +55,11 @@ resources:
             wc_notify --data-binary ''{"status": "SUCCESS", "reason":
             "signal4", "data": "data4"}''
 
-            # check signals with the same number
+            # check signals with the same ID
 
-            wc_notify --data-binary ''{"status": "SUCCESS", "id": "5"}''
+            wc_notify --data-binary ''{"status": "SUCCESS", "id": "test5"}''
 
-            wc_notify --data-binary ''{"status": "SUCCESS", "id": "5"}''
+            wc_notify --data-binary ''{"status": "SUCCESS", "id": "test5"}''
 
             # loop for 20 signals without reasons and data
 
@@ -76,7 +76,7 @@ resources:
     type: OS::Heat::WaitCondition
     depends_on: instance1
     properties:
-      count: 12
+      count: 25
       handle: {get_resource: wait_handle}
       timeout: {get_param: timeout}
 
